"""
This will work best if readline and rlcompleter are installed.  Then
there will be Chicken Scheme completion when in a Chicken REPL and
Python completion when in a Python REPL.
"""
from scyth cimport *
from os.path import expanduser, isfile
from re import search as re_search
from atexit import register as atexit_register

# ********************
#
# WHY DOES (define x 20) PRINT OUT 20?
#
#**********************
    
######################################################################
#    Global Variables
#
#  See below for the CHICKEN_KEYWORDS dictionary
#
LINE_NUMBER = 1
SETUP_DONE = 0
IN_STR = list ()
HOLD = ""
######################################################################

######################################################################
# This is the main command
def c_(once = 0):
    """
    Evaluates a Chicken Scheme expression.
    """
    global LINE_NUMBER
    global IN_STR
    global HOLD
    
    # x == 1 means evaluate on expression and return results
    # This is setting up our Scheme environment the way we want it
    # It is only done the first time we invoke the Chicken interpreter
    if (SETUP_DONE == 0):
        c_setup ()
        # Set up the first Chicken completion history
        c_first_completion()
    else:
        # Here we need to close the Python completion history first
        c_completion()
        inc_line_number()

    # We'll either do one evaluation and return to Python...
    if (once == 1):
        as_once = 0
    #... or we'll do multiple evaluations until we (exit) or ,q.
    elif (once == 0):
        as_once = 1

    # Start evaluating the Chicken input
    # I put prompt here because if I put it as an argument it
    # printed #;1> everytime we reentered Chicken
    prompt="#;" + str(LINE_NUMBER) + "> "
    while True:
        hold_str1 = raw_input (prompt)
        tester = ("".join (hold_str1.split ()))
        
        # Accidental double entry
        if ((not IN_STR) and (not tester)):
            print "No command entered."
            prompt = "#;" + str(LINE_NUMBER) + "> "

        # Evaluate the Scheme expression using a newline
        elif (not tester):
            eval_with_newline ()
            if (once == 1):
                return HOLD
            else:
                prompt = "#;" + str (LINE_NUMBER) + "> "
                
        # The following will only work if IN_STR is empty.
        # I'll need to change this sometime.
        elif (not IN_STR):
            # Exit the Scheme REPL
            if ((tester == "(exit)") or (tester == ",q")):
                return exit_called ()
            else:
                prompt = eval_empty_in_str (hold_str1)
            
        else:
            prompt = eval_multiline (hold_str1)
######################################################################


def add_to_globals(string1):
    """
    Adds a variable to the Python globals().
    """
    holder = split_string(string1)
    # Check if it's a definition
    if (holder[0] == "define"):
        # Make sure it's a variable assignement instead of
        # a function definition.  We'll do this by checking
        # for a ( after the define statement        
        if (holder[1][0] != "("):
            try:
                # This gives the value we'll assign to the variable
                dict_hold = chick_eval_s_to_s_no_print (holder[2])
            except:
                pass
            else:
                try:
                    eval_hold = eval (dict_hold)
                    globals()[holder[1]] = eval_hold
                # We have a string
                except ValueError:
                    globals()[holder[1]] = dict_hold
                # I don't know what this is
                except:
                    globals()[holder[1]] = None
        # This is probably a function definition
        else:
            # We've only got a ( here, so pick the next word
            if (holder[1] == "("):
                new_name = holder[2].split(None,1)                
                globals()[new_name[0]] = None
            # Otherwise strip off the (
            else:
                globals()[holder[1][1:]] = None
###########################################            
def c_completion():
    """
    Sets up the completion of Chicken keywords
    """
    global CHICKEN_KEYWORDS
    # Set up our Chicken completion environment
    try:
        import readline
    except ImportError:
        pass
    else:
        # Write to the Python REPL history, which is open
        historyPath = expanduser("~/scythians/.pychick.history")
        if isfile(historyPath):
            readline.write_history_file(historyPath)
            readline.clear_history()
        
        # Open the Chicken REPL history file
        historyPath = expanduser("~/scythians/.chickpy.history")
        if isfile(historyPath):
            readline.read_history_file(historyPath)
###########################################
def c_first_completion():
    """
    Sets up the completion of Chicken keywords
    """
    global CHICKEN_KEYWORDS
    # Add Chicken keywords to Python's global dictionary
    for k, v in CHICKEN_KEYWORDS.items():
        globals()[k] = v
    
    # Set up our Chicken completion environment
    try:
        import readline
    except ImportError:
        pass
    else:
        import rlcompleter
        readline.parse_and_bind("tab: complete")
        readline.set_history_length(250)

        # Open the Chicken REPL history file
        # The Python REPL history is not open
        historyPath = expanduser("~/scythians/.chickpy.history")
        if isfile(historyPath):
            readline.read_history_file(historyPath)
###########################################
def c_setup ():
    """
    Set up the Chicken Scheme environment.
    """
    global SETUP_DONE
    print "\nCHICKEN"
    print "(c)2008-2010 The Chicken Team"
    print "(c)2000-2007 Felix L. Winkelmann"
    CHICKEN_run (CHICKEN_default_toplevel)
    home_path = expanduser("~/")
    if isfile(home_path + ".csirc"):
        print "\nLoading " + home_path + ".csirc."
        load_str = '(load "' + home_path + '.csirc")'            
        chick_eval_s_to_s(load_str)
    inc_setup_done ()
    print "\nThe output of the Chicken Scheme REPL is assigned to the Python variable first_output.\n"
###########################################    
def check_str (string1):
    try:
        indx = re_search(';.*?;', string1).span ()
        indx1 = min (indx) + 1
        indx2 = max (indx) - 1
        print string1[indx1:indx2] + " is not defined."
        return True
    except:
        return False
###########################################    
def chick_eval_s_to_s (string1):
    return C_chick_eval_s_to_s(string1)
###########################################    
def chick_eval_s_to_s_no_print (string1):
    return C_chick_eval_s_to_s_no_print(string1)
###########################################
def eval_empty_in_str (hold_str1):
    # We know that hold_str1 is not empty since
    # (not tester) == False.  We'll split hold_str1
    # into the command, the first argument, and
    # everything else that we're going to evaluate
    word_list = split_string (hold_str1)
    # Make sure a command was entered
    if (len (word_list) == 0):
        print "No command was entered."
        return "#;" + str (LINE_NUMBER) + "> "
        
    test_word = word_list [0]
    
    # A toplevel command other that ,q
    if (test_word [0] == ","):
        run_toplevel (word_list)
        return "#;" + str (LINE_NUMBER) + "> "

    # A multi-line expression
    else:
        return eval_multiline (hold_str1)
###########################################    
def eval_multiline (string1):
    global IN_STR
    hold_str2 = parse_once (string1)
    # Check if a keyword is a valid Python variable
    if (check_str (hold_str2)):
        IN_STR = list()
        return "#;" + str(LINE_NUMBER) + "> "
    # Otherwise we have a multi-line expression
    else:
        IN_STR.append(hold_str2)
        return " --> "
###########################################    
def eval_with_newline ():
    global LINE_NUMBER
    global IN_STR
    global HOLD
    eval_str = " ".join (IN_STR)    
    HOLD = chick_eval_s_to_s (eval_str)
    # Update Python's globals() so we can
    # access definitions later    
    add_to_globals(eval_str)
    inc_line_number ()
    IN_STR = list ()
###########################################    
def exit_called ():
    global HOLD
    print "Process scheme finished"

    try:
        import readline
    except ImportError:
        pass
    else:
        # Write to the Chicken REPL history file
        historyPath = expanduser("~/scythians/.chickpy.history")
        if isfile(historyPath):
            readline.write_history_file(historyPath)
            readline.clear_history()
         
    # Set up Python's completion
    p_completion()
    
    # No output to send back?
    if (str(HOLD) == '#<unspecified>'):
        return 
    else:
        # See if we have a number
        try:
            eval_HOLD = eval (HOLD)
            return eval_HOLD
        # We have a string
        except ValueError:
            return HOLD
        #EOF
        except:
            return
###########################################        
def inc_line_number ():
    global LINE_NUMBER
    LINE_NUMBER += 1
###########################################    
def inc_setup_done ():
    global SETUP_DONE
    SETUP_DONE = 1
###########################################    
def p_completion():
    """
    Sets up Python's completion environment
    """
    global CHICKEN_KEYWORDS
    # Set up our Python completion environment
    try:
        import readline
    except:
        pass
    else:
        # Open the Python REPL history file
        historyPath = expanduser("~/scythians/.pychick.history")
        if isfile(historyPath):
            readline.read_history_file(historyPath)
###########################################            
def parse_once (string1):
    """
    Replaces keywords with their Python values.
    """
    try:
        indx = re_search(';.*?;', string1).span ()
    except:
        return string1
    indx1 = min (indx) + 1
    indx2 = max (indx) - 1
    try_evaluate = ""
    try:
        try_evaluate = eval (string1 [indx1:indx2])        
    except:        
        return string1 [indx1 - 1:indx2 + 1]
    # We have a number
    try:
        if (try_evaluate == try_evaluate - 0):
            string1 = string1 [:indx1 - 1] + " " + \
                str (try_evaluate) + \
                " " + parse_once (string1 [indx2 + 1:])
    # Otherwise we need to quote it (_NOT_ Scheme (quote .... ) though)
    except:
        string1 = string1 [:indx1 - 1] + \
            ' "' + str (try_evaluate) + '" ' + \
            parse_once (string1 [indx2 + 1:])        
    return string1
###########################################
def run_toplevel (word_list):
    """
    Runs a Chicken toplevel command.
    """
    global LINE_NUMBER
    if (word_list [0] == ",l"):
        for xx in word_list [1:]:
            print "loading " + xx
            chick_eval_s_to_s ('(load "' + xx + '")')
        inc_line_number ()
    else:
        print "Not a toplevel command."
###########################################    
def split_string (hold_str1):
    # Remove all of the leading and trailing whitespace
    holder = hold_str1.strip ()
    # check if we have a leading parentheses
    if (holder [0] == "("):
        # Remove the leading and trailing parentheses
        holder2 = holder [1:-1]
    else:
        # We'll be printing a number or some such thing
        holder2 = holder
    # py_strig is the second word and chick_string is
    # everything else without any trailing whitespace
    return holder2.split (None,2)
###########################################
###########################################    

CHICKEN_KEYWORDS = dict({'define':None, \
                             'lambda':None, \
                             'exit':None})
    





	
    

